@@ -9,6 +9,7 @@ from collections import OrderedDict
 import docutils
 import docutils.core
 import docutils.io
+from docutils.parsers.rst.languages import get_language as get_docutils_lang
 from docutils.writers.html4css1 import HTMLTranslator, Writer
 
 import six
@@ -218,6 +219,14 @@ class RstReader(BaseReader):
     def __init__(self, *args, **kwargs):
         super(RstReader, self).__init__(*args, **kwargs)
 
+        lang_code = self.settings.get('DEFAULT_LANG', 'en')
+        if get_docutils_lang(lang_code):
+            self._language_code = lang_code
+        else:
+            logger.warning("Docutils has no localization for '%s'."
+                           " Using 'en' instead.", lang_code)
+            self._language_code = 'en'
+
     def _parse_metadata(self, document, source_path):
         """Return the dict containing document metadata"""
         formatted_fields = self.settings['FORMATTED_FIELDS']
@@ -256,7 +265,7 @@ class RstReader(BaseReader):
         extra_params = {'initial_header_level': '2',
                         'syntax_highlight': 'short',
                         'input_encoding': 'utf-8',
-                        'language_code': self.settings.get('DEFAULT_LANG'),
+                        'language_code': self._language_code,
                         'halt_level': 2,
                         'traceback': True,
                         'warning_stream': StringIO(),
@@ -358,7 +367,7 @@ class HTMLReader(BaseReader):
     class _HTMLParser(HTMLParser):
         def __init__(self, settings, filename):
             try:
-                # Python 3.4+
+                # Python 3.5+
                 HTMLParser.__init__(self, convert_charrefs=False)
             except TypeError:
                 HTMLParser.__init__(self)
@@ -689,8 +698,20 @@ def path_metadata(full_path, source_path, settings=None):
         if settings.get('DEFAULT_DATE', None) == 'fs':
             metadata['date'] = SafeDatetime.fromtimestamp(
                 os.stat(full_path).st_mtime)
-        metadata.update(settings.get('EXTRA_PATH_METADATA', {}).get(
-            source_path, {}))
+
+        # Apply EXTRA_PATH_METADATA for the source path and the paths of any
+        # parent directories. Sorting EPM first ensures that the most specific
+        # path wins conflicts.
+
+        epm = settings.get('EXTRA_PATH_METADATA', {})
+        for path, meta in sorted(epm.items()):
+            # Enforce a trailing slash when checking for parent directories.
+            # This prevents false positives when one file or directory's name
+            # is a prefix of another's.
+            dirpath = os.path.join(path, '')
+            if source_path == path or source_path.startswith(dirpath):
+                metadata.update(meta)
+
     return metadata
 
 
